/***************************************************************************
 *   Copyright (C) 2005 by Peter Donis                                     *
 *   peterdonis@alum.mit.edu                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <kmainwindow.h>
#include <klocale.h>
#include <kmessagebox.h>

#include "kidserver.h"

kidserver::kidserver(const KAboutData *aboutdata, const char *url)
  : KMainWindow( 0, "kidserver" )
{
  idserver_running = FALSE;

  /* Create UI widget and set caption from it */
  widget = new kidserverWidget( this );
  resize(widget->width(), widget->height());
  setCentralWidget( widget );
  //setCaption( widget->caption() );

  /* Connect widget signals to our slots */
  connect(widget, SIGNAL(kPushButtonAbout_clicked()), this, SLOT(widget_clickedAbout()));
  connect(widget, SIGNAL(kPushButtonQuit_clicked()), this, SLOT(widget_clickedQuit()));
  connect(widget, SIGNAL(kPushButtonGo_clicked()), this, SLOT(widget_clickedGo()));
  connect(widget, SIGNAL(kPushButtonStop_clicked()), this, SLOT(widget_clickedStop()));

  /* Create about box */
  aboutBox = new KAboutApplication(aboutdata, this, "About KIDServer");

  /* Create idserver process manager */
  idserver = new KProcess();

  /* Connect process signals to our slots */
  connect(idserver, SIGNAL(processExited(KProcess *)),
    this, SLOT(idserver_exited(KProcess *)));
  connect(idserver, SIGNAL(receivedStdout(KProcess *, char *, int)),
    this, SLOT(idserver_output(KProcess *, char *, int)));
  connect(idserver, SIGNAL(receivedStderr(KProcess *, char *, int)),
    this, SLOT(idserver_output(KProcess *, char *, int)));
  
  /* if URL was passed on the command line, display and check it */
  if ( url )
  {
    widget->lineEditURL_setText(url);
    go_url();
  }
}

kidserver::~kidserver()
{
  /* Clean up */
  disconnect(idserver, SIGNAL(processExited(KProcess *)),
    this, SLOT(idserver_exited(KProcess *)));
  disconnect(idserver, SIGNAL(receivedStdout(KProcess *, char *, int)),
    this, SLOT(idserver_output(KProcess *, char *, int)));
  disconnect(idserver, SIGNAL(receivedStderr(KProcess *, char *, int)),
    this, SLOT(idserver_output(KProcess *, char *, int)));
  delete idserver;

  delete aboutBox;

  disconnect(widget, SIGNAL(kPushButtonAbout_clicked()), this, SLOT(widget_clickedAbout()));
  disconnect(widget, SIGNAL(kPushButtonQuit_clicked()), this, SLOT(widget_clickedQuit()));
  disconnect(widget, SIGNAL(kPushButtonGo_clicked()), this, SLOT(widget_clickedGo()));
  disconnect(widget, SIGNAL(kPushButtonStop_clicked()), this, SLOT(widget_clickedStop()));
  delete widget;
}

void kidserver::widget_clickedAbout()
{
  aboutBox->show();
}

void kidserver::widget_clickedQuit()
{
  close();
}

void kidserver::widget_clickedGo()
{
  go_url();
}

void kidserver::widget_clickedStop()
{
  stop_url();
}

static const char *sorryMsg =
I18N_NOOP("The IDserver program must be installed for kidserver to work properly.\n\
You can find IDserver on the same web site where you found kidserver.");

void kidserver::go_url()
{
  /* Only run one instance at a time */
  if ( !idserver_running ) {

    /* Get command line for idserver process from widget fields */
    idserver->clearArguments();
    *idserver << "idserver";
    if ( widget->DNS_only() ) {
      *idserver << "-l";
    } else {
      if ( widget->protocol_set() ) *idserver << "-p" << widget->protocol_arg();
      if ( widget->port_set() ) *idserver << "-r" << widget->port_arg();
    }
    *idserver << widget->URL_text();
  
    /* Clear widget output viewer */
    widget->kTextBrowserOutput_clear();

    /* Start the idserver process -- it will run concurrently and its output to stdout
       (or stderr) will be sent to the idserver_output slot below */
    if ( idserver->start(KProcess::NotifyOnExit, KProcess::AllOutput) ) {
      /* Set field indicating that idserver is running */
      idserver_running = TRUE;
      /* Tell widget idserver is running */
      widget->proc_running();
    } else {
      /* IDserver wasn't found (this is the only likely reason we would
         ever get here, since idserver is such a small process) */
      KMessageBox::detailedSorry(this, I18N_NOOP("IDserver not found."), sorryMsg);
    }
  }
}

void kidserver::idserver_output(KProcess *, char *buffer, int)
{
  /* Display the output from idserver in the widget viewer */
  widget->kTextBrowserOutput_output(buffer);
}

void kidserver::idserver_exited(KProcess *)
{
  /* Set field indicating idserver process is done */
  idserver_running = FALSE;
  /* Tell widget idserver is done */
  widget->proc_stopped();
}

void kidserver::stop_url()
{
  /* Only stop if idserver is running */
  if ( idserver_running ) {
    if ( idserver->kill() ) {
      /* We won't get a "process exited" signal so fake it */
      idserver_exited(idserver);
      /* Output message to the widget */
      widget->kTextBrowserOutput_output("Stopped idserver.\n");
    } else {
      /* Couldn't stop the process */
      KMessageBox::sorry(this, I18N_NOOP("Could not stop idserver."));
    }
  }
}

#include "kidserver.moc"
